from flask import request, session, url_for
from requests_oauthlib import OAuth2Session


class OAuth2Login(object):

  def __init__(self, app=None):
    if app:
      self.init_app(app)
      self.app = app

  def get_config(self, app, name, default_value=None):
    return app.config.get(self.config_prefix + name, default_value)

  def init_app(self, app):
    self.client_id = self.get_config(app, "CLIENT_ID")
    self.client_secret = self.get_config(app, "CLIENT_SECRET")
    self.scope = self.get_config(app, "SCOPE", self.default_scope).split(",")
    self.redirect_scheme = self.get_config(app, "REDIRECT_SCHEME", "https")

    app.add_url_rule(
      self.get_config(app, "REDIRECT_PATH", self.default_redirect_path),
      self.redirect_endpoint,
      self.login,
    )

  @property
  def redirect_uri(self):
    return url_for(
      self.redirect_endpoint,
      _external=True,
      _scheme=self.redirect_scheme,
    )

  def session(self):
    return OAuth2Session(
      self.client_id,
      redirect_uri=self.redirect_uri,
      scope=self.scope,
    )

  def authorization_url(self, **kwargs):
    sess = self.session()
    auth_url, state = sess.authorization_url(self.auth_url, **kwargs)
    session[self.state_session_key] = state
    return auth_url

  def login(self):
    sess = self.session()
    try:
      sess.fetch_token(
        self.token_url,
        code=request.args["code"],
        client_secret=self.client_secret,
      )
      profile = self.get_profile(sess)
      # TODO: Check state
    except Exception as e:
      return self.login_failure_func(e)
    else:
      return self.login_success_func(sess.token, profile)

  def login_success(self, f):
    self.login_success_func = f
    return f

  def login_failure(self, f):
    self.login_failure_func = f
    return f

  def get_profile(self, sess):
    raise NotImplementedError




from .base import OAuth2Login


class CourseraLogin(OAuth2Login):

  config_prefix = "COURSERA_LOGIN_"
  redirect_endpoint = "_coursera_login"
  state_session_key = "_coursera_login_state"

  default_scope = "view_profile"
  default_redirect_path = "/login/coursera"

  auth_url = "https://accounts.coursera.org/oauth2/v1/auth"
  token_url = "https://accounts.coursera.org/oauth2/v1/token"
  profile_url = "https://api.coursera.org/api/externalBasicProfiles.v1"

  def get_profile(self, sess):
    resp = sess.get(
      self.profile_url,
      params=dict(
        q="me",
        fields=",".join([
          "name",
        ]),
      )
    )
    resp.raise_for_status()
    return resp.json()["elements"][0]




from .base import OAuth2Login


class LinkedInLogin(OAuth2Login):

  config_prefix = "LINKEDIN_LOGIN"
  redirect_endpoint = "_linkedin_login"
  state_session_key = "_linkedin_login_state"

  default_scope = (
    "r_basicprofile",
    "r_emailaddress",
  )
  default_redirect_path = "/login/linkedin"

  auth_url = "https://www.linkedin.com/uas/oauth2/authorization"
  token_url = "https://www.linkedin.com/uas/oauth2/accessToken"
  profile_url = "https://api.linkedin.com/v1/people/~"

  def get_profile(self, sess):
    resp = sess.get(self.profile_url)
    resp.raise_for_status()
    return resp.json()




from setuptools import setup, find_packages


if __name__ == "__main__":
  setup(
    name="Flask-OAuth2-Login",
    version="0.0.4",
    packages=find_packages(),
    install_requires=[
      "requests",
      "requests-oauthlib",
    ],
    author="Mark Steve Samson",
    author_email="hello@marksteve.com",
    description="Simple OAuth2 login",
    license="MIT",
    url="https://github.com/marksteve/flask-oauth2-login",
  )



from .base import OAuth2Login


class GoogleLogin(OAuth2Login):

  config_prefix = "GOOGLE_LOGIN_"
  redirect_endpoint = "_google_login"
  state_session_key = "_google_login_state"

  default_scope = (
    "https://www.googleapis.com/auth/userinfo.email,"
    "https://www.googleapis.com/auth/userinfo.profile"
  )
  default_redirect_path = "/login/google"

  auth_url = "https://accounts.google.com/o/oauth2/auth"
  token_url = "https://accounts.google.com/o/oauth2/token"
  profile_url = "https://www.googleapis.com/oauth2/v2/userinfo"

  def get_profile(self, sess):
    resp = sess.get(self.profile_url)
    # FIXME: Check `error` key
    resp.raise_for_status()
    return resp.json()

